//
// NetworkSetupController.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.Linq;
using Ntp.Analyzer.Gui.Window;
using Ntp.Analyzer.Gui.Model;
using Ntp.Analyzer.Gui.Transaction;
using Ntp.Analyzer.Gui.Data;

namespace Ntp.Analyzer.Gui.Control
{
    public class NetworkSetupController
    {
        public NetworkSetupController (MasterController master)
        {
            //this.master = master;
            dialog = new NetworkDialog (this);
            transaction = new HostTransaction ();
        }

        //private readonly MasterController master;
        private readonly NetworkDialog dialog;
        private readonly HostTransaction transaction;

        public void Start ()
        {
            foreach (Host host in HostDataStore.Instance.Hosts) {
                dialog.AppendLine (host.Id, host.Name, host.Address, host.Port);
            }

            dialog.ShowAll ();
        }
        #region Controller logic
        public void Add ()
        {
            Host host = new Host ();
            dialog.AppendLine (host.Id, host.Name, host.Address, host.Port);

            transaction.Add (new HostAddition (host));
        }

        public void DeleteRow (int id)
        {
            Host host = HostDataStore.Instance.Hosts.SingleOrDefault (h => h.Id == id);

            if (host == null) {
                transaction.Remove (id);
            } else {
                transaction.Add (new HostRemoval (host));
            }
        }

        public void CellUpdated (int id, int propIndex, string value)
        {
            Host host = HostDataStore.Instance.Hosts.SingleOrDefault (h => h.Id == id);

            if (host == null) {
                host = transaction.Cache.Single (h => h.Id == id);
            }

            transaction.Add (new HostUpdate (host, propIndex, value));
        }

        public void ApplySelected ()
        {
            transaction.Commit ();
            dialog.HideAll ();
        }

        public void CancelSelected ()
        {
            transaction.Abort ();
            dialog.HideAll ();
        }
        #endregion
    }
}
