//
// HostTransaction.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.Collections.Generic;
using Ntp.Analyzer.Gui.Model;

namespace Ntp.Analyzer.Gui.Transaction
{
    public sealed class HostTransaction
    {
        public HostTransaction ()
        {
            operations = new List<HostOperation> ();
        }

        private List<HostOperation> operations;

        public IEnumerable<Host> Cache {
            get {
                List<Host> cache = new List<Host> ();

                foreach (HostOperation operation in operations) {
                    if (!cache.Contains (operation.Host)) {
                        cache.Add (operation.Host);
                    }
                }

                return cache;
            }
        }

        public void Add (HostOperation operation)
        {
            operations.Add (operation);
        }

        public void Remove (int id)
        {
            List<HostOperation> toDelete = new List<HostOperation> ();

            foreach (HostOperation operation in operations) {
                if (operation.HostId == id) {
                    toDelete.Add (operation);
                }
            }

            foreach (HostOperation operation in toDelete) {
                operations.Remove (operation);
            }
        }

        public void Commit ()
        {
            foreach (HostOperation operation in operations) {
                operation.Execute ();
            }
        }

        public void Abort ()
        {
            operations.Clear ();
        }
    }
}
