//
// FileCollection.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using Ntp.Analyzer.Export;
using Ntp.Analyzer.Localize;
using Ntp.Config;

namespace Ntp.Analyzer.Config.Destination
{
    public sealed class FileCollection : DestinationCollection
    {
        private FileCollection(
            string configName,
            int indentLevel,
            IEnumerable<FileStreamDestination> destinations)
            : base(configName, indentLevel)
        {
            this.destinations = destinations;
        }

        public const string Identifier = ConfigurationKeyword.FileDestinations;
        private readonly IEnumerable<FileStreamDestination> destinations;

        public override IEnumerable<StreamDestination> Destinations
        {
            get { return destinations; }
        }

        public override string ToString()
        {
            var values = new List<Object> {null};
            IEnumerable<String> names = Destinations.Select(f => f.Value);
            values.AddRange(names);

            var settings = new List<string> {Identifier};

            int num = destinations.Count();

            for (int i = 0; i < num; i++)
                settings.Add(ConfigurationKeyword.FileItem);

            return ConfigText(settings.ToArray(), values.ToArray());
        }

        public static FileCollection Create(
            HostConfiguration server,
            ConfigurationBlock config)
        {
            var files = new List<FileStreamDestination>();

            ConfigurationBlock block = FindSingleBlock(Identifier, config);

            if (block == null)
                return new FileCollection(String.Empty, config.IndentLevel + 1, files);

            foreach (string destination in GetMultiOption(ConfigurationKeyword.FileItem, block))
                files.Add(new FileStreamDestination(server.FilePath, destination));

            return new FileCollection(String.Empty, config.IndentLevel + 1, files);
        }
    }
}