//
// HostSubConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using Ntp.Analyzer.Import;
using Ntp.Process;

namespace Ntp.Analyzer.Config
{
    /// <summary>
    /// Configuration class for subsections of a <see cref="Ntp.Analyzer.Config.HostConfiguration" />.
    /// </summary>
    public abstract class HostSubConfiguration : JobConfiguration
    {
        protected HostSubConfiguration(
            HostConfiguration server,
            string configName,
            int indentLevel,
            int frequency,
            bool initialRun,
            bool fixedRun)
            : base(configName, indentLevel, frequency, initialRun, fixedRun)
        {
            this.server = server;
        }

        private readonly HostConfiguration server;

        /// <summary>
        /// Gets the name of the server in the <see cref="Ntp.Analyzer.Config.HostSubConfiguration" />.
        /// </summary>
        /// <value>The name of the server.</value>
        public string ServerName
        {
            get { return server.ServerName; }
        }

        /// <summary>
        /// Gets the type of the server in the <see cref="Ntp.Analyzer.Config.HostSubConfiguration" />.
        /// </summary>
        /// <value>The type of the server.</value>
        public ServerType ServerType
        {
            get { return server.ServerType; }
        }

        /// <summary>
        /// Gets the host identifier in the <see cref="Ntp.Analyzer.Config.HostSubConfiguration" />.
        /// </summary>
        /// <value>The host identifier.</value>
        public int HostId
        {
            get { return server.HostId; }
        }

        /// <summary>
        /// Gets the file path in the <see cref="Ntp.Analyzer.Config.HostSubConfiguration" />.
        /// </summary>
        /// <value>The web path.</value>
        public string FilePath
        {
            get { return server.FilePath; }
        }

        /// <summary>
        /// Gets the web path in the <see cref="Ntp.Analyzer.Config.HostSubConfiguration" />.
        /// </summary>
        /// <value>The web path.</value>
        public string WebPath
        {
            get { return server.WebPath; }
        }
    }
}