//
// MenuConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using Ntp.Analyzer.Localize;
using Ntp.Config;

namespace Ntp.Analyzer.Config.Navigation
{
    public sealed class MenuConfiguration : ConfigurationBase
    {
        private MenuConfiguration(
            string configName,
            int indentLevel,
            IEnumerable<MenuItemConfiguration> menuItems)
            : base(configName, indentLevel)
        {
            this.menuItems = new List<MenuItemConfiguration>(menuItems);
        }

        public const string Identifier = ConfigurationKeyword.Menu;

        private static readonly String[] Params =
        {
            ConfigurationKeyword.Menu,
            MenuItemConfiguration.Identifier
        };

        private readonly List<MenuItemConfiguration> menuItems;

        public IEnumerable<MenuItemConfiguration> MenuItems
        {
            get { return menuItems; }
        }

        protected override IEnumerable<Object> Items
        {
            get
            {
                return new object[]
                {
                    ConfigName,
                    menuItems
                };
            }
        }

        public override string ToString()
        {
            return ConfigText(Params, Items);
        }

        public static MenuConfiguration Create(ConfigurationBlock config, IEnumerable<ILinkable> linkables)
        {
            Dictionary<String, String> options = GetOptions(Params, config);

            string name = GetOptionConfigName(
                options, ConfigurationKeyword.Menu, Identifier, false, Identifier);

            ConfigurationBlock[] itemBlocks = FindBlocks(ConfigurationKeyword.MenuItem, config);
            List<MenuItemConfiguration> items = itemBlocks.
                Select(itemBlock => MenuItemConfiguration.Create(itemBlock, linkables)).
                ToList();

            return new MenuConfiguration(name, config.IndentLevel, items);
        }
    }
}