//
// PageConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Linq;
using Ntp.Analyzer.Config.Destination;
using Ntp.Analyzer.Localize;

namespace Ntp.Analyzer.Config.Page
{
    public abstract class PageConfigurationBase : HostSubConfiguration
    {
        protected PageConfigurationBase(
            HostConfiguration server,
            string configName,
            int indentLevel,
            int frequency,
            bool initialRun,
            bool fixedRun,
            string theme,
            string title,
            DestinationCollection destinations,
            string link)
            : base(server, configName, indentLevel, frequency, initialRun, fixedRun)
        {
            this.theme = theme;
            this.title = title;
            this.destinations = destinations;
            this.link = link;
        }

        protected const bool DefaultInitialRun = true;
        protected const bool DefaultFixedRun = true;
        protected const string DefaultTheme = ConfigurationKeyword.PageTemplateDefault;
        protected const string DefaultTitle = "";
        protected const int DefaultLinkIndex = 1;
        protected readonly DestinationCollection destinations;
        private readonly string link;
        private readonly string theme;
        private readonly string title;

        /// <summary>
        /// Gets the name of this <see cref="Ntp.Analyzer.Config.Navigation.ILinkable" />.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get { return ConfigName; }
        }

        /// <summary>
        /// Gets the theme/template to use for this page.
        /// </summary>
        /// <value>The theme.</value>
        public string Theme
        {
            get { return theme.ToLower(); }
        }

        /// <summary>
        /// Gets the title of this page.
        /// </summary>
        /// <value>The title.</value>
        public string Title
        {
            get { return title; }
        }

        /// <summary>
        /// Gets the destinations for output generated by this configuration.
        /// </summary>
        /// <value>The destinations.</value>
        public DestinationCollection Destinations
        {
            get { return destinations; }
        }

        /// <summary>
        /// Gets the link to this page.
        /// </summary>
        /// <value>The link.</value>
        public string Link
        {
            get { return link != null ? link : DefaultLink; }
        }

        protected string DefaultLink
        {
            get { return Destinations.First().Location.Replace(FilePath, String.Empty); }
        }
    }
}