//
// ClusterConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Ntp.Analyzer.Localize;
using Ntp.Config;

namespace Ntp.Analyzer.Config.Root
{
    public sealed class ClusterConfiguration : ConfigurationBase
    {
        private ClusterConfiguration(
            string name,
            int indentLevel,
            IEnumerable<NodeConfiguration> nodes)
            : base(name, indentLevel)
        {
            this.nodes = nodes;
        }

        public const string Identifier = ConfigurationKeyword.Cluster;

        private static readonly String[] Params =
        {
            ConfigurationKeyword.Cluster,
            NodeConfiguration.Identifier
        };

        private readonly IEnumerable<NodeConfiguration> nodes;

        public IEnumerable<NodeConfiguration> Nodes
        {
            get { return nodes; }
        }

        protected override IEnumerable<Object> Items
        {
            get
            {
                return new object[]
                {
                    ConfigName,
                    Nodes
                };
            }
        }

        public static ClusterConfiguration Create(ConfigurationBlock config)
        {
            if (config == null)
                return null;

            OptionSet options = GetOptions(Params, config);

            var nodes = new List<NodeConfiguration>();

            // TODO: Implement validation
            foreach (ConfigurationBlock nodeBlock in FindBlocks(NodeConfiguration.Identifier, config))
            {
                NodeConfiguration node = NodeConfiguration.Create(nodeBlock);

                if (node != null)
                {
                    nodes.Add(node);
                }
            }

            return new ClusterConfiguration(
                GetOptionConfigName(options, Identifier, Identifier, true),
                config.IndentLevel,
                nodes);
        }
    }
}