﻿//
// SqlDatabaseFactory.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Data.Common;

namespace Ntp.Data.Provider
{
    public abstract class SqlDatabaseFactory : ISqlFactory
    {
        protected static DatabaseConfiguration Config;

        public static SqlDatabaseFactory Instance
        {
            get
            {
                switch (Config.Provider)
                {
                    case SqlDatabaseProvider.MySql:
#if MYSQL
                        return new MySqlFactory();
#else
                        throw new ApplicationException("NTP Analyzer was not build wit MySQL support.");
#endif
                    case SqlDatabaseProvider.PostgreSql:
#if PGSQL
                        return new PostgreSqlFactory();
#else
                        throw new ApplicationException("NTP Analyzer was not build wit PostgreSQL support.");
#endif
                    default:
                        return null;
                }
            }
        }

        public abstract string PrepareCheckTableSql(string table);

        public abstract DbConnection CreateConnection();

        public abstract DbConnection CreateGenericConnection();

        public abstract DbCommand CreateCommand();

        public abstract DbParameter CreateParameter();

        public abstract DbParameter CreateParameter(string name, object value);

        public static void Initialize(DatabaseConfiguration factoryConfiguration)
        {
            Config = factoryConfiguration;
        }

        public abstract string PrepareSql(string sql);
        public abstract string PrepareCreateTableSql(string sql);

        public abstract string PrepareInsertSql(string sql);

        public abstract void CreateDatabase();
    }
}