﻿//
// DirectoryCommand.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System.IO;

#if __MonoCS__
using System;
using System.Collections.Generic;
using System.Linq;
#endif

namespace Ntp.Interop
{
    public static class DirectoryCommand
    {
        public static string CreateRecursiveFromFile(string file)
        {
            int pos = file.LastIndexOf('/');
            if (pos == -1)
                return null;

            string path = file.Substring(0, pos);
            return CreateRecursive(path);
        }

#if __MonoCS__
        public static string CreateRecursive(string path)
        {
            string fullPath = path.Trim ();
            if (fullPath.Length <= 1)
                return null;

            string created = null;
            char[] s = new char[] { '/' };
            List<string> elements = fullPath.Split (s, StringSplitOptions.RemoveEmptyEntries).ToList ();

            string currentPath = String.Empty;
            foreach (string element in elements) {
                currentPath = currentPath + "/" + element;
                if (!Directory.Exists (currentPath)) {
                    Directory.CreateDirectory (currentPath);
                    created = currentPath;
                }
            }

            return created;
        }

        #else
        public static string CreateRecursive(string path)
        {
            Directory.CreateDirectory(path);
            return null;
        }
#endif
    }
}