//
// LogGroup.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013-2016 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace Ntp.Log
{
    public sealed class LogGroup : LogBase, IEnumerable<LogBase>
    {
        internal LogGroup()
            : base(Severity.Trace)
        {
            logs = new List<LogBase>();
        }

        private readonly List<LogBase> logs;

        private LogBase SysLog
        {
            get
            {
                return
                    logs.
                        Where(l => l is SysLog).
                        OrderBy(l => l.Treshold).
                        FirstOrDefault();
            }
        }

        private LogBase ConsoleLog
        {
            get
            {
                return
                    logs.
                        Where(l => l is ConsoleLog).
                        OrderBy(l => l.Treshold).
                        FirstOrDefault();
            }
        }

        public IEnumerator<LogBase> GetEnumerator()
        {
            return logs.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        public override void Initialize()
        {
            foreach (LogBase log in logs)
                log.Initialize();
        }

        public override void Close()
        {
            foreach (LogBase log in logs)
                log.Close();
        }

        public override void Suspend()
        {
            foreach (LogBase log in logs)
                log.Suspend();
        }

        public override void Resume()
        {
            foreach (LogBase log in logs)
                log.Resume();
        }

        public override void WriteLine(string text, Severity severity)
        {
            if (SysLog != null)
                SysLog.WriteLine(text, severity);

            if (ConsoleLog != null)
                ConsoleLog.WriteLine(text, severity);

            foreach (LogBase log in logs.Where(l => l is FileLog))
                log.WriteLine(text, severity);
        }

        public override void WriteLine(Exception exception)
        {
            if (SysLog != null)
                SysLog.WriteLine(exception);

            if (ConsoleLog != null)
                ConsoleLog.WriteLine(exception);

            foreach (LogBase log in logs.Where(l => l is FileLog))
                log.WriteLine(exception);
        }

        public override void WriteLine(Exception exception, Severity severity)
        {
            if (SysLog != null)
                SysLog.WriteLine(exception, severity);

            if (ConsoleLog != null)
                ConsoleLog.WriteLine(exception, severity);

            foreach (LogBase log in logs.Where(l => l is FileLog))
                log.WriteLine(exception, severity);
        }

        public void Add(LogBase log)
        {
            if (log is LogGroup)
            {
                logs.AddRange(log as LogGroup);
            }
            else
            {
                logs.Add(log);
            }
        }
    }
}